#include "includes.h"
#include "Console_Terminal.h"

CommandLine Commands[] = 
{
//   Key           Name        ParamNum
  {_NOP,          "NOP",          0},   // wrong input line
  {_EMPTY,        "",             0},   // empty line
  {_HELP,         "HELP",         1},   // shows a help info on the terminal
  {_CLS,          "CLS",          0},   // clear display
  {_P,            "P",            3},   // draws a point on the display
  {_L,            "L",            5},   // draws a line on the display
  {_T,            "T",            7},   // draws a triangle on the display
  {_TF,           "TF",           8},   // draws a triangle on the display filled inside
  {_R,            "R",            5},   // draws a rectangle on the display
  {_RF,           "RF",           6},   // draws a rectangle on the display filled inside
  {_E,            "E",            5},   // draws a ellipse on the display
  {_C,            "C",            4},   // draws a circle on the display
  {_FILES,        "FILES",        0},   // shows a files in the current directory
  {_CHDIR,        "CHDIR",        1},   // changes the current directory
  {_DISPLAY,      "DISPLAY",      1},   // draws the picture on the display
  {_DISPLAY_BOX,  "DISPLAY_BOX",  5},   // draws the picture on the display in a box
  {_SLIDE,        "SLIDE",       -1},   // slideshow the selected pictures with some delay
  {_CLR,          "CLR",          0},   // clear terminal
  {_BAUD,         "BAUD",         1},   // set BaudRate
  {_TOUCH_SENS,   "TOUCH_SENS",   1},   // set touch screen sensitivity
};

int NumberCommands = ARRAY_NUMELEM(Commands);

/*******************************************************************************
 * Function Name: NewLine
 * Parameters:  None
 *
 * Return: None
 *
 * Desrpition: Print a new line at the console terminal.
 *
 ******************************************************************************/
void NewLine (void)
{
  int x;
  x = strlen ("\r\n");
  UartWrite(UART_0,"\r\n",x);
  return;
}

/*******************************************************************************
 * Function Name: ReadLn
 * Parameters:  Line - the string that keep the input string
 *
 * Return: int 0 - no new line(enter)
 *             1 - new line (enter is pressed).
 *
 * Desrpition: Continiously reading symbols from the UART0 and put them into a 
 *             <Buffer> and when enter is pressed all the bytes are copied to
 *             <Line> (the input parameter). In that case the next input starts
 *             from the begining.
 *
 ******************************************************************************/

int ReadLn (char Line[])
{
  static Int8U Buffer[256];
  static int Length = 0, Size;
  
  // Data from UART0
  Size = UartRead(UART_0,Buffer+Length,sizeof(Buffer)-1);
  if (Size)
  {
    //if (Buffer[Length] == 13)
    if (Buffer[Length] == '\r')
    {
      int IndexLine = 0;
      for (int IndexBuffer=0; IndexBuffer<Length; IndexBuffer++)
      {
        if (Buffer[IndexBuffer] != 0x7F)
          Line[IndexLine++] = Buffer[IndexBuffer];
        else
        {
          if (IndexLine > 0)    // check if there are ANY symbols to be removed
            IndexLine--;
        }
      }
      Line[IndexLine] = 0;
      Length = 0;
      NewLine ();
      return 1;
    }
    Length = Length + Size;;
  }
  return 0;
}

/*******************************************************************************
 * Function Name: Write
 * Parameters:  Line - the string to be printed.
 *
 * Return: None
 *
 * Desrpition: Print the string on the console terminal.
 *
 ******************************************************************************/
void Write (char Line[])
{
  pInt8U pBuffer = (unsigned char *) Line;
  int TranSize = 0, Size = strlen (Line);
  do
  {
    Size -= TranSize;
    pBuffer += TranSize;
    TranSize = UartWrite(UART_0,pBuffer,Size);
  }
  while(Size > TranSize); 
  return;
}

/*******************************************************************************
 * Function Name: WriteLn
 * Parameters:  Line - the string to be printed.
 *
 * Return: None
 *
 * Desrpition:  Print the string on the console terminal and add a new line at
 *              the end of the text.
 *
 ******************************************************************************/
void WriteLn (char Line[])
{
  Write (Line);
  NewLine ();
  return;
}

CommandKey Form_Command (char Input[], int *argc, char ***argv)
{
  char Cmd[MAX_COMMAND_LENGTH];
  int i, CommandIndex, ArgIndex=0, First, Last, ArgLen;
  CommandKey Key = _NOP;
  
  // count number of arguments on the input
  *argc = 0;
  for (i=0; Input[i]; i++)
    if (Input[i] == ' ')
      if (Input[i-1] != ' ')
        (*argc)++;
  
  // check for the last symbol if it is a part of the last argument (different of ' ')
  if (i>0)
    if (Input[i-1] != ' ')
      (*argc)++;
  
  // allocate memory for loading the arguments in the memory
  *(argv) = (char**) malloc ((*argc) * sizeof (char*));
  
  // get the first argument of the input (command)
  for (i=0; Input[i]; i++)
    if (Input[i] == ' ' || Input[i] == '\n' || Input[i] == '\r')  //separators
      break;
    else
      Cmd[i] = Input[i];
  
  Cmd[i] = 0;

  // check for an empty line
  if (i==0)
  {
    *argc = 0;
    return _EMPTY;
  }
  
  // compare the name of the command with the list of commands
  for (CommandIndex=0; CommandIndex<ARRAY_NUMELEM(Commands); CommandIndex++)
    if (!My_StrCmp (Cmd, Commands[CommandIndex].Name, NO_CASE_SENS))
    {
      Key = Commands[CommandIndex].Key;
      break;
    }
  
  // check for inexisting command
  if (Key == _NOP)
  {
    *argc = 0;
    return _NOP;
  }
  
  // check for incorrect number of parameters
  if (Commands[CommandIndex].ParamNum != -1 && Commands[CommandIndex].ParamNum != (*argc)-1)
  {
    *argc = 0;
    return _INVALID;
  }
  
  // allocate heap-memory for the first parameter (command name) and store it there
  ArgLen = strlen (Cmd);
  *(*(argv) + ArgIndex) = (char*) malloc (ArgLen * sizeof(char) + 1);
  strcpy ((**(argv)) + ArgIndex, Cmd);

  // check if the command has no parameters
  if (Input[i]==0)    // if ((*argc)-1 == 0)  //TODO 
    return Key;
  
  // allocate memory for the list of arguments and store them in the heap-memory
  ArgIndex = 1;
  First = i;

  for (i=First+1; Input[i]; i++)
  {
    if (Input[i] == ' ')
      if (Input[i-1] == ' ')
      {
        First++;
        continue;
      }
      else
      {
        Last = i;

        *(*(argv) + ArgIndex) = (char*) malloc ((Last-First) * sizeof (char)); // this includes byte for \0
        if (*(*(argv) + ArgIndex) == NULL)
        {
          Write ("No memory!\n\r");
          break;
        }
        String_Copy_From_To (*(*(argv)+ArgIndex), Input, First + 1, Last - 1);
        *(*(*(argv) + ArgIndex) + (Last - First - 1)) = 0;    // \0
        First = Last;
        ArgIndex++;
      }
  }
  // this is for the last parameter on the line
  Last = i;
  *(*(argv) + ArgIndex) = (char*) malloc ((Last-First) * sizeof (char)); // this includes byte for \0
  if (*(*(argv) + ArgIndex) == NULL)
    Write ("No memory!\n\r");
  String_Copy_From_To (*(*(argv)+ArgIndex), Input, First + 1, Last - 1);
  *(*(*(argv) + ArgIndex) + (Last - First - 1)) = 0;    // \0
  
  return Key;   // the key of the command-name
}

void Dispatch_Command (CommandKey Key, int argc, char **argv)
{
  switch (Key)
  {
  case _INVALID:
    Invalid_Parameters ();
    break;
    
  case _NOP:
    No_Operation ();
    break;
    
  case _EMPTY:
    Empty_Line ();
    break;
    
  case _HELP:
    Help (argv[1]);
    break;
    
  case _CLS:
    Clear_Screen ();
    break;
    
  case _P:
    {
      int X, Y;
      long int Color;
      X = Convert_String_to_Int (argv[1]);
      Y = Convert_String_to_Int (argv[2]);
      Color = Convert_String_to_Int (argv[3]);
      Draw_Point (X, Y, Color);
      Synchronize_Buffers ();
    }
    break;
    
  case _L:
    {
      int X1, Y1, X2, Y2;
      long int Color;
      X1 = Convert_String_to_Int (argv[1]);
      Y1 = Convert_String_to_Int (argv[2]);
      X2 = Convert_String_to_Int (argv[3]);
      Y2 = Convert_String_to_Int (argv[4]);
      Color = Convert_String_to_Int (argv[5]);
      Draw_Line (X1, Y1, X2, Y2, Color);
      Synchronize_Buffers ();
    }
    break;
    
  case _T:
    {
      int X1, Y1, X2, Y2, X3, Y3;
      long int Color;
      X1 = Convert_String_to_Int (argv[1]);
      Y1 = Convert_String_to_Int (argv[2]);
      X2 = Convert_String_to_Int (argv[3]);
      Y2 = Convert_String_to_Int (argv[4]);
      X3 = Convert_String_to_Int (argv[5]);
      Y3 = Convert_String_to_Int (argv[6]);
      Color = Convert_String_to_Int (argv[7]);
      Draw_Triangle (X1, Y1, X2, Y2, X3, Y3, Color);
      Synchronize_Buffers ();
    }
    break;
  
  case _TF:
    {
      int X1, Y1, X2, Y2, X3, Y3;
      long int Color, ColorBackground;
      X1 = Convert_String_to_Int (argv[1]);
      Y1 = Convert_String_to_Int (argv[2]);
      X2 = Convert_String_to_Int (argv[3]);
      Y2 = Convert_String_to_Int (argv[4]);
      X3 = Convert_String_to_Int (argv[5]);
      Y3 = Convert_String_to_Int (argv[6]);
      Color = Convert_String_to_Int (argv[7]);
      ColorBackground = Convert_String_to_Int (argv[8]);
      Switch_Buffers ();
      Fill_Triangle (X1, Y1, X2, Y2, X3, Y3, ColorBackground);
      Draw_Triangle (X1, Y1, X2, Y2, X3, Y3, Color);
      Switch_to_Active_Buffer ();
      Synchronize_Buffers ();
      Switch_Buffers ();
      Switch_to_Active_Buffer ();
    }
    break;
    
  case _R:
    {
      int X1, Y1, X2, Y2;
      long int Color;
      X1 = Convert_String_to_Int (argv[1]);
      Y1 = Convert_String_to_Int (argv[2]);
      X2 = Convert_String_to_Int (argv[3]);
      Y2 = Convert_String_to_Int (argv[4]);
      Color = Convert_String_to_Int (argv[5]);
      Draw_Rectangle (X1, Y1, X2, Y2, Color);
      Synchronize_Buffers ();
    }
    break;

  case _RF:
    {
      int X1, Y1, X2, Y2;
      long int Color, ColorBackground;
      X1 = Convert_String_to_Int (argv[1]);
      Y1 = Convert_String_to_Int (argv[2]);
      X2 = Convert_String_to_Int (argv[3]);
      Y2 = Convert_String_to_Int (argv[4]);
      Color = Convert_String_to_Int (argv[5]);
      ColorBackground = Convert_String_to_Int (argv[6]);
      Switch_Buffers ();
      Fill_Rectangle (X1, Y1, X2, Y2, ColorBackground);
      Draw_Rectangle (X1, Y1, X2, Y2, Color);
      Switch_to_Active_Buffer ();
      Synchronize_Buffers ();
      Switch_Buffers ();
      Switch_to_Active_Buffer ();
    }
    break;
    
  case _E:
    {
      int X, Y, R1, R2;
      long int Color;
      X = Convert_String_to_Int (argv[1]);
      Y = Convert_String_to_Int (argv[2]);
      R1 = Convert_String_to_Int (argv[3]);
      R2 = Convert_String_to_Int (argv[4]);
      Color = Convert_String_to_Int (argv[5]);
      Draw_Ellipse (X, Y, R1, R2, Color);
      Synchronize_Buffers ();
    }
    break;
    
  case _C:
    {
      int X, Y, R;
      long int Color;
      X = Convert_String_to_Int (argv[1]);
      Y = Convert_String_to_Int (argv[2]);
      R = Convert_String_to_Int (argv[3]);
      Color = Convert_String_to_Int (argv[4]);
      Draw_Circle (X, Y, R, Color);
      Synchronize_Buffers ();
    }
    break;
    
  case _FILES:
    {
      if (SDCardError)
        Write ("SD card problem detected!\n\r");
      else
        Files ();
    }
    break;
    
  case _CHDIR:
    {
      if (SDCardError)
        Write ("SD card problem detected!\n\r");
      else
        Change_Directory (argv[1]);
    }
    break;
    
  case _DISPLAY:
    {
      if (SDCardError)
        Write ("SD card problem detected!\n\r");
      else
        Display (argv[1]);
    }
    break;
  case _DISPLAY_BOX:
    {
      Point UL, DR;
      int Clr;
      if (SDCardError)
        Write ("SD card problem detected!\n\r");
      else
      {
        UL.X = Convert_String_to_Int (argv[2]);
        UL.Y = Convert_String_to_Int (argv[3]);
        DR.X = Convert_String_to_Int (argv[4]);
        DR.Y = Convert_String_to_Int (argv[5]);
        Clr = Convert_String_to_Int (argv[6]);
        Display_Box (argv[1], UL, DR, Clr);
      }
    }
    break;
    
  case _SLIDE:
    {
      unsigned long int Delay;
      if (SDCardError)
        Write ("SD card problem detected!\n\r");
      else
      {
        Delay = Convert_String_to_Int (argv[1]);
        Slide (Delay, argc, argv);
      }
    }
    break;
    
  case _CLR:
    Write("\f");
    break;
    
  case _BAUD:
    {
      int Baud;
      Baud = Convert_String_to_Int (argv[1]);
      if (Baud > 0 && Baud < 4 && Baud != BaudRateKey)
      {
        Write ("RESET WITH NEW BAUDRATE!\n\r");
        BaudRateKey = Baud;
        for (unsigned int i=65000; i; i--);
      }
      switch (Baud)
      {
      case 1:
        UART_Configurate (9600);
        break;
      case 2:
        UART_Configurate (19200);
        break;
      case 3:
        UART_Configurate (115200);
        break;
      default:
        Write ("Wrong parameter value!\n\r");
        break;
      }
    }
    break;
    
  case _TOUCH_SENS:
    Touch_Threshold = Convert_String_to_Int (argv[1]);
    break;
    
  default:
    break;
  }
}
